/*
 * Copyright 2021-2024 Avaiga Private Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */

/**************************************************************

                   TYPOGRAPHY UTILITIES

***************************************************************/

.text-weight300 {
    font-weight: 300 !important;
}

.text-weight400 {
    font-weight: 400 !important;
}

.text-weight500 {
    font-weight: 500 !important;
}

.text-weight600 {
    font-weight: 600 !important;
}

.text-weight700 {
    font-weight: 700 !important;
}

.text-weight800 {
    font-weight: 800 !important;
}

.text-weight900 {
    font-weight: 900 !important;
}

.text-left {
    text-align: left !important;
}

.text-center {
    text-align: center !important;
}

.text-right {
    text-align: right !important;
}

.text-uppercase {
    text-transform: uppercase !important;
}

.text-no-transform {
    text-transform: none !important;
}

.text-underline {
    text-decoration: underline !important;
}

.text-no-underline {
    text-decoration: none !important;
}
