/*! For license information please see taipy-gui-dom.js.LICENSE.txt */
(()=>{var e={338:(e,r,t)=>{"use strict";var o=t(879);r.H=o.createRoot,o.hydrateRoot},20:(e,r,t)=>{"use strict";var o=t(922),s=Symbol.for("react.element"),n=(Symbol.for("react.fragment"),Object.prototype.hasOwnProperty),a=o.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED.ReactCurrentOwner,i={key:!0,ref:!0,__self:!0,__source:!0};r.jsx=function(e,r,t){var o,p={},u=null,c=null;for(o in void 0!==t&&(u=""+t),void 0!==r.key&&(u=""+r.key),void 0!==r.ref&&(c=r.ref),r)n.call(r,o)&&!i.hasOwnProperty(o)&&(p[o]=r[o]);if(e&&e.defaultProps)for(o in r=e.defaultProps)void 0===p[o]&&(p[o]=r[o]);return{$$typeof:s,type:e,key:u,ref:c,props:p,_owner:a.current}}},848:(e,r,t)=>{"use strict";e.exports=t(20)},922:(e,r,t)=>{e.exports=t(942)(6540)},879:(e,r,t)=>{e.exports=t(942)(961)},942:e=>{"use strict";e.exports=TaipyGuiDependencies}},r={};function t(o){var s=r[o];if(void 0!==s)return s.exports;var n=r[o]={exports:{}};return e[o](n,n.exports,t),n.exports}(()=>{"use strict";var e=t(848),r=t(338);const o=TaipyGui;var s=document.getElementById("root");s&&(0,r.H)(s).render((0,e.jsx)(o.Router,{}))})()})();